import java.applet.Applet;
import java.awt.*;
import jv.object.*;
import jv.viewer.PvViewer;
import jv.project.PvDisplayIf;
import jv.thirdParty.ruler.PgAxes;
import jv.vecmath.PdVector;

/**
 * Applet shows a bcc lattice in real and reciprocal space.
 * Source code is availabe at bottom of page.
 * 
 *
 * @author Simon Gamper
 * @version 14.02.2011, 1.0
 */
public class BCCApplet extends Applet implements Runnable {
	/**
	 * Extend Applet class of Java
	 */
	/**
	 * Message string drawn in applet while loading. Modify string with
	 * drawMessage().
	 */
	private String m_drawString = "Initializing ...";

	private static final long serialVersionUID = 1L;
	/**
	 * If variable m_frame!=null then this applet runs in an application,
	 * otherwise applet runs inside an Html page within a browser. Further, when
	 * running as application then this frame is the container of the applet.
	 */
	public Frame m_frame = null;
	/**
	 * 3D-viewer window for graphics output, display is embedded into the
	 * applet. This instance variable allows JavaView to perform start, stop and
	 * destroy operations when the corresponding methods of this applet are
	 * called, for example, by a browser.
	 */
	protected PvViewer m_viewer;

	/**
	 * Configure and initialize the viewer, load system and user projects. One
	 * of the user projects must be selected here.
	 */
	public void init() {
		Thread thread = new Thread(this, "JavaView: inititialize applet");
		thread.setPriority(Thread.NORM_PRIORITY);
		thread.start();
	}

	public void run() {
		// Create viewer for viewing 3d geometries. References to the applet and
		// frame
		// allow JavaView to decide whether program runs as applet or standalone
		// application,
		// and, in the later case, it allows to use the frame as parent frame.
		drawMessage("Loading viewer ...");
		m_viewer = new PvViewer(this, m_frame);
		//m_viewer.showDialog(36);// show help dialog
		drawMessage("Loading geometry ...");
		// Create and load a project which contains the user application.
		// Putting code
		// in a JavaView project allows to reuse the project in other
		// applications.
		BccProject myProject = new BccProject();
		m_viewer.addProject(myProject);
		m_viewer.selectProject(myProject);

		// Get 3d display from viewer and add it to applet
		setLayout(new BorderLayout());
		PvDisplayIf disp = m_viewer.getDisplay();
		add((Component) disp, BorderLayout.CENTER);
		disp.showAxes(true);
		PgAxes axes = ((jv.viewer.PvDisplay) disp).getAxes();
		axes.setNames(new String[] { "k\u2081", "k\u2082", "k\u2083" });
		axes.update(axes);
		add(m_viewer.getPanel(PsViewerIf.PROJECT), BorderLayout.EAST);
		disp.getCamera().setDist(10);
		disp.getCamera().setRoll(-180.);
		disp.getCamera().setViewDir(new PdVector(-.5,-.5,-.5));
		validate();
		// Choose initial panel in control window (must press F1 inside the
		// applet)
		// m_viewer.showPanel(PsViewerIf.MATERIAL);

		startFromThread();
	}

	/**
	 * Standalone application support. The main() method acts as the applet's
	 * entry point when it is run as a standalone application. It is ignored if
	 * the applet is run from within an HTML page.
	 */
	public static void main(String args[]) {
		BCCApplet app = new BCCApplet();
		// Create toplevel window of application containing the applet
		Frame frame = new jv.object.PsMainFrame(app, args);
		frame.pack();
		// Store the variable frame inside the applet to indicate
		// that this applet runs as application.
		app.m_frame = frame;
		app.init();
		// In application mode, explicitly call the applet.start() method.
		app.start();
		// Set size of frame when running as application.
		frame.setSize(640, 550);
		frame.setVisible(true);
	}

	public String getAppletInfo() {
		return "Name: "
				+ this.getClass().getName()
				+ "\r\n"
				+ "Author: "
				+ "Simon Gamper"
				+ "\r\n"
				+ "Version: "
				+ "1.00 feb/2011"
				+ "\r\n"
				+ "Applet shows visualization of bcc lattice in real and reciprocal space.Source code is availabe at bottom of page."
				+ "\r\n";
	}

	/** Set message string to be drawn in apllet while loading. */
	private void drawMessage(String message) {
		m_drawString = message;
		repaint();
	}

	public void paint(Graphics g) {
		g.setColor(Color.blue);
		g.drawString(PsConfig.getProgramAndVersion(), 20, 40);
		g.drawString(m_drawString, 20, 60);
	}

	/**
	 * Does clean-up when applet is destroyed by the browser. Here we just close
	 * and dispose all our control windows.
	 */
	public void destroy() {
		m_viewer.destroy();
	}

	/** Stop viewer, e.g. stop animation if requested */
	public void stop() {
		m_viewer.stop();
	}

	/** Replace start() method of Applet */
	public void startFromThread() {
		if (m_viewer != null)
			m_viewer.start();
	}
}
