import jv.number.PuDouble;
import jv.project.PjProject;
import jv.vecmath.PdVector;
import jv.object.PsDebug;
import jv.project.PvDisplayIf;
import jv.thirdParty.ruler.PgAxes;


/**
 * Applet shows WignerSeitz cell and first brioullin zone of a bcc lattice. It is
 * written with the Javaview library. For more information visit
 * http://www.javaview.de.
 * 
 * @see	jv.project.PjProject
 * @author Simon Gamper
 * @version 14.02.2011, 1.0
 */

public class BccProject extends PjProject implements Runnable {
	/**
	 * Hallo
	 */
	private static final long serialVersionUID = 1L;
	protected RealLattice m_real; // class real
	// protected ReciprocalLattice m_reciprocal;
	protected ReciprocalLattice m_reciprocal;// class reciprocal
	protected boolean m_ShowPrimitive_a; // Variable to set primitive cell
											// visible
	protected boolean m_ShowLattice_a; // Variable to set lattice points visible
	protected boolean m_ShowSymmetry_a; // Variable to set symmetry points
										// visible
	protected boolean m_ShowKVectors_a; // Variable to set tranlation vectors
										// visible
	protected boolean m_ShowEuclidian_a;// Variable to set euclidian geometry
										// visible
	protected double m_distance_gamma_n;// Variable of measured distance
	protected double m_distance_gamma_h;// Variable of measured distance
	protected double m_distance_gamma_p;// Variable of measured distance
	protected transient Thread thread;// Thread to handle autorotation
	protected PuDouble m_a; // Variable to handel slider lattice constant
	protected String[] m_types = { "Real Space", "Reciprocal Space" }; // string
																		// array
																		// to
																		// define
																		// choice
																		// types
	protected String m_deftype = m_types[1];// default choice type
	protected String m_typeName;// string to save current state of choice
	protected boolean del=true;
	/**
	 * Constructor, without arguments to allow loading of project from menu.
	 */
	public BccProject() {
		super("MyProject"); // Call super class PJPROJECT
		m_typeName = m_deftype;

		m_real = new RealLattice(); // constructor of real geometries


		m_reciprocal = new ReciprocalLattice(); // constructor of reciprocal
												// geometries

		m_a = new PuDouble("Lattice constant a", this); // set parent of slider
		if (getClass() == BccProject.class)
			init(); // Call to init() is required.
	}

	/**
	 * Do initialization of data structures; method is also used to reset
	 * instances.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see jv.project.PjProject#init()
	 */
	public void init() {
		super.init();// call superclass init
		

		m_ShowPrimitive_a = false;
		

		
		
		// Set values of visibility
		m_ShowPrimitive_a = false;
		m_ShowKVectors_a = false;
		m_ShowLattice_a = true;
		m_ShowSymmetry_a = true;
		m_ShowEuclidian_a = true;
		// Initialize integers bounds and values of slider.
		m_a.setDefBounds(.1, 50, 0.1, 2);
		m_a.setDefValue(2.4);
		m_a.init();
		// call init of reciprocal lattice
		m_reciprocal.init(m_a.getValue());
		m_reciprocal.computePoly(m_a.getValue());
		m_reciprocal.computebrioullin(m_a.getValue());
		// call init of reciprocal lattice
		m_real.init(m_a.getValue());
		m_real.computePoly(m_a.getValue());
		m_real.computebrioullin(m_a.getValue());
		// set default display geometries
		if (m_typeName.startsWith("Real")) {
			addGeometry(m_real.m_geom);// adding geometry to the display
			addGeometry(m_real.m_element);// adding geometry to the display
			addGeometry(m_real.m_brioullin);// adding geometry to the display
			selectGeometry(m_real.m_geom);// select active geometry
		} else if (m_typeName.startsWith("Reciprocal")) {
			addGeometry(m_reciprocal.m_geom);// adding geometry to the display
			addGeometry(m_reciprocal.m_brioullin);// adding geometry to the
													// display
			addGeometry(m_reciprocal.m_element);// adding geometry to the
												// display
			addGeometry(m_reciprocal.m_symmetry);// adding geometry to the
													// display
			selectGeometry(m_reciprocal.m_brioullin);// select active geometry
		}
	
	}

	/**
	 * Start project, e.g. start an animation. Method is called once when
	 * project is selected in PvViewer#selectProject(). Method is optional. For
	 * example, if an applet calls the start() method of PvViewer, then PvViewer
	 * tries to invoke the start() method of the currently selected project.
	 */
	public void start() {
		// show help on start

		// start auto animation
		if (PsDebug.NOTIFY)
			PsDebug.notify("anim.start: ");
		PvDisplayIf disp = getDisplay();
		if (disp != null) {
			disp.setEnabledAnimation(true);
			disp.setAutoRotation(new PdVector(1., 0., 1.), .01);
		}
		thread = new Thread(this, "JavaView");
		thread.start();
		super.start();
		
	
	}

	public void run() {
	}// is needed but not used

	public void stopAnim() { // stop auto animation
		PvDisplayIf disp = getDisplay();
		disp.setEnabledAnimation(false);
	}

	/**
	 * update current objects e.g. geometries changed in instrument panel
	 */
	public boolean update(Object event) {
		if (event == getInfoPanel() || event == m_a) {
			m_real.computePoly(m_a.getValue()); // recalculate with new lattice
												// constant value from slider
			m_real.computebrioullin(m_a.getValue());// recalculate with new
													// lattice constant value
													// from slider
			// update geometries
			m_real.m_geom.update(m_real.m_geom);
			m_real.m_element.update(m_real.m_element);
			m_real.m_edge.update(m_real.m_edge);
			m_real.m_kdir.update(m_real.m_kdir);
			m_real.m_brioullin.update(m_real.m_brioullin);
			m_real.m_symmetry.update(m_real.m_symmetry);

			m_reciprocal.computePoly(m_a.getValue()); // recalculate with new
														// lattice constant
														// value from slider
			m_reciprocal.computebrioullin(m_a.getValue()); // recalculate with
															// new lattice
															// constant value
															// from slider
			m_reciprocal.m_geom.update(m_reciprocal.m_geom);
			m_reciprocal.m_element.update(m_reciprocal.m_element);
			m_reciprocal.m_edge.update(m_reciprocal.m_edge);
			m_reciprocal.m_kdir.update(m_reciprocal.m_kdir);
			m_reciprocal.m_brioullin.update(m_reciprocal.m_brioullin);
			m_reciprocal.m_symmetry.update(m_reciprocal.m_symmetry);
			return super.update(this);

		}
		return super.update(event); // return other unknown event to superclass
	}

	public String gettypeName() {
		return m_typeName;
	}// method to get choice state

	public void settypeName(String typeName)// method to set choice type
	{
		m_typeName = typeName.trim();// trim vanish spaces before and after
										// string
	}

	public void updateGeometry(String typeName) {// update geometries in frame
													// when choice changes
		if (m_typeName.startsWith("Real")) {
			removeGeometries();// clear frame
			PgAxes axes = ((jv.viewer.PvDisplay) getDisplay()).getAxes();// set
																			// axes
																			// label
			axes.setNames(new String[] { "x\u2081", "x\u2082", "x\u2083" });
			axes.update(axes);// necessary to see changes
			// set "active" geometries
			if (m_ShowLattice_a)
				addGeometry(m_real.m_geom);
			addGeometry(m_real.m_element);
			addGeometry(m_real.m_brioullin);
			selectGeometry(m_real.m_brioullin);
			addGeometry(m_real.m_symmetry);
		} else if (m_typeName.startsWith("Reciprocal")) {
			removeGeometries();// clear frame
			PgAxes axes = ((jv.viewer.PvDisplay) getDisplay()).getAxes();// set
																			// axes
																			// label
			axes.setNames(new String[] { "k\u2081", "k\u2082", "k\u2083" });
			axes.update(axes);// necessary to see changes
			// set "active" geometries
			if (m_ShowLattice_a)
				addGeometry(m_reciprocal.m_geom);
			addGeometry(m_reciprocal.m_brioullin);
			addGeometry(m_reciprocal.m_element);
			addGeometry(m_reciprocal.m_symmetry);
			selectGeometry(m_reciprocal.m_brioullin);
		}
	}

}
