import java.awt.Color;
import jv.geom.PgPointSet;
import jv.geom.PgPolygonSet;
import jv.vecmath.PiVector;
import jv.vecmath.PdVector;

public class RealLattice extends PgPointSet {
	/**
	 * Class to generate geometries of real bcc lattice for bcc applet
	 * 
	 * @see	PgPointSet
	 * @author Simon Gamper
	 * @version 14.02.2011, 1.0
	 */
	private static final long serialVersionUID = 1L;
	protected PgPointSet m_geom; // Pointset of geometry
	protected PgPointSet m_element;
	protected PgPolygonSet m_edge;
	protected PgPolygonSet m_kdir;
	protected PgPolygonSet m_brioullin;
	protected PgPolygonSet m_symmetry;
	protected int m_numUPoints = 5;
	protected int m_numVPoints = 5;
	protected int m_numWPoints = 5;
	protected double m_Distance_gamma_1;
	protected double m_Distance_gamma_2;
	protected double m_Distance_gamma_3;
	protected double m_Distance_gamma_4;
	protected double m_Distance_gamma_5;
	protected double m_Distance_gamma_6;

	public RealLattice() {

		m_geom = new PgPointSet(3);
		m_geom.setName("Lattice");

		m_element = new PgPointSet(3);
		m_element.setName("Brioullin");

		m_edge = new PgPolygonSet(3);
		m_edge.setName("Primitive Cell");

		m_kdir = new PgPolygonSet(3);
		m_kdir.setName("k-Diretion vectors");

		m_brioullin = new PgPolygonSet(3);
		m_brioullin.setName("brioullin zone");

		m_symmetry = new PgPolygonSet(3);
		m_symmetry.setName("symmetry points");
		init();
	}

	public void init(double a) {

		m_brioullin.setNumVertices(24);
		m_brioullin.setGlobalVertexSize(1.);
		m_brioullin.setGlobalVertexColor(Color.green);
		m_brioullin.setNumPolygons(1);
		m_brioullin.setVertex(0, a / 4, 3 * a / 4, a / 2);
		m_brioullin.setVertex(1, a / 4, a / 2, a * 3 / 4);
		m_brioullin.setVertex(2, -a / 4, a / 4, a / 2);
		m_brioullin.setVertex(3, 0, -a / 2, 0);
		m_brioullin.setVertex(4, 0, 0, a / 2);
		m_brioullin.setVertex(5, 0, 0, -a / 2);
		m_brioullin.setVertex(6, a / 4, a / 4, a / 4);
		m_brioullin.setVertex(7, -a / 4, a / 4, a / 4);
		m_brioullin.setVertex(8, a / 4, -a / 4, a / 4);
		m_brioullin.setVertex(9, a / 4, a / 4, -a / 4);
		m_brioullin.setVertex(10, -a / 4, -a / 4, a / 4);
		m_brioullin.setVertex(11, -a / 4, a / 4, -a / 4);
		m_brioullin.setVertex(12, a / 4, -a / 4, -a / 4);
		m_brioullin.setVertex(13, -a / 4, -a / 4, -a / 4);

		int xa[] = { 0, 1, 2, 3, 0, 8, 11, 21, 20, 3, 2, 12, 15, 23, 20, 21,
				22, 23, 15, 14, 6, 7, 22, 23, 22, 7, 4, 10, 11, 8, 9, 10, 4, 5,
				18, 17, 9, 17, 16, 1, 16, 19, 13, 12, 13, 14, 6, 5, 18, 19 };
		PiVector le = new PiVector(xa);
		m_brioullin.setPolygon(0, le);
		m_brioullin.setGlobalPolygonColor(Color.green);

		m_kdir.setNumVertices(4);
		m_kdir.setNumPolygons(3);
		m_kdir.setVertex(0, 0, 0, 0);
		m_kdir.setPolygon(0, new PiVector(0, 1));
		m_kdir.getPolygon(0).setName("b1");
		m_kdir.setPolygon(1, new PiVector(0, 2));
		m_kdir.getPolygon(1).setName("b2");
		m_kdir.setPolygon(2, new PiVector(0, 3));
		m_kdir.getPolygon(2).setName("b3");
		m_kdir.setVertex(3, -a / 2, a / 2, a / 2);
		m_kdir.setVertex(1, a / 2, -a / 2, a / 2);
		m_kdir.setVertex(2, a / 2, a / 2, -a / 2);
		m_kdir.setGlobalPolygonColor(Color.blue);
		m_kdir.showPolygonEndArrow(true);
		m_kdir.showPolygonLabels(true);

		// init symmetry points
		m_symmetry.setNumVertices(6);

		m_symmetry.setVertex(0, new PdVector(0, 0, 0));
		m_symmetry.getVertex(0).setName("\u0393");
		PdVector X = PdVector.blendNew(0.5, m_kdir.getVertex(2), 0.5,
				m_kdir.getVertex(3));
		m_symmetry.setVertex(1, X);
		m_symmetry.getVertex(1).setName("X");
		PdVector L = PdVector.blendNew(0.5, m_kdir.getVertex(1), 0.5,
				m_kdir.getVertex(2), 0.5, m_kdir.getVertex(3));
		m_symmetry.setVertex(2, L);
		m_symmetry.getVertex(2).setName("L");
		PdVector W = PdVector.blendNew(0.25, m_kdir.getVertex(1), 0.75,
				m_kdir.getVertex(2), 0.5, m_kdir.getVertex(3));
		m_symmetry.setVertex(3, W);
		m_symmetry.getVertex(3).setName("W");
		PdVector U = PdVector.blendNew(1 / 4, m_kdir.getVertex(1), 5 / 8,
				m_kdir.getVertex(2), 5 / 8, m_kdir.getVertex(3));
		m_symmetry.setVertex(4, U);
		m_symmetry.getVertex(4).setName("U");
		PdVector K = new PdVector(3);
		K = PdVector.blendNew(3 / 8, m_kdir.getVertex(1), 3 / 4,
				m_kdir.getVertex(2), 3 / 8, m_kdir.getVertex(3));
		m_symmetry.setVertex(5, K);
		m_symmetry.getVertex(5).setName("K");

		m_symmetry.showVertexLabels(true);
		m_symmetry.showEdgeLabels(true);
		m_symmetry.setNumPolygons(9);
		m_symmetry.setPolygon(0, new PiVector(0, 5));
		m_symmetry.getPolygon(0).setName("\u03A3");
		m_symmetry.setPolygon(1, new PiVector(0, 1));
		m_symmetry.getPolygon(1).setName("\u039B");
		m_symmetry.setPolygon(2, new PiVector(3, 5));
		m_symmetry.getPolygon(2).setName("");
		m_symmetry.setPolygon(3, new PiVector(1, 3));
		m_symmetry.getPolygon(3).setName("Z");
		m_symmetry.setPolygon(4, new PiVector(1, 4));
		m_symmetry.getPolygon(4).setName("S");
		m_symmetry.setPolygon(5, new PiVector(2, 4));
		m_symmetry.getPolygon(5).setName("");
		m_symmetry.setPolygon(6, new PiVector(2, 3));
		m_symmetry.getPolygon(6).setName("Q");
		m_symmetry.setPolygon(7, new PiVector(2, 5));
		m_symmetry.getPolygon(7).setName("");
		m_symmetry.setPolygon(8, new PiVector(0, 2));
		m_symmetry.getPolygon(8).setName("\u0394");

		m_geom.setGlobalVertexColor(Color.ORANGE);
		m_geom.setNumVertices(m_numUPoints * m_numVPoints * m_numWPoints);
		m_geom.setVertexColor(
				(m_numUPoints * m_numVPoints * m_numWPoints + 1) / 2,
				Color.yellow);

		m_edge.setGlobalVertexColor(Color.yellow);
		m_edge.setGlobalVertexSize(4.);
		m_edge.setGlobalVertexSize(3.);
		computePoly(a); // Compute initial surface
		m_element.setGlobalVertexSize(1.);
		computebrioullin(a);

	}

	public void computePoly(double b) {
		// Compute lattice points
		double a = b;
		PdVector x = new PdVector(-a / 2, a / 2, a / 2);
		PdVector y = new PdVector(a / 2, -a / 2, a / 2);
		PdVector z = new PdVector(a / 2, a / 2, -a / 2);
		PdVector t = new PdVector(3);

		int ind = 0;
		int length = (int) Math.floor(m_numUPoints / 2);
		for (int i = -length; i <= length; i++) {
			for (int j = -length; j <= length; j++) {
				for (int k = -length; k <= length; k++) {
					t = PdVector.blendNew(i, x, j, y, k, z);
					m_geom.setVertex(ind, t);
					ind++;
				}
			}
		}
	}

	public void computebrioullin(double b) {
		// compute points of lattice
		double a = b;// 2*Math.PI/b;

		m_element.setNumVertices(29);
		m_edge.setNumPolygons(6);
		m_edge.setNumVertices(30);

		m_kdir.setVertex(3, -a / 2, a / 2, a / 2);
		m_kdir.setVertex(1, a / 2, -a / 2, a / 2);
		m_kdir.setVertex(2, a / 2, a / 2, -a / 2);

		// compute points of brioullin zone
		m_brioullin.setVertex(0, a / 2, 0, a / 4);
		m_brioullin.setVertex(1, a / 2, a / 4, 0);
		m_brioullin.setVertex(2, a / 2, 0, -a / 4);
		m_brioullin.setVertex(3, a / 2, -a / 4, 0);

		m_brioullin.setVertex(4, -a / 2, 0, a / 4);
		m_brioullin.setVertex(5, -a / 2, a / 4, 0);
		m_brioullin.setVertex(6, -a / 2, 0, -a / 4);
		m_brioullin.setVertex(7, -a / 2, -a / 4, 0);

		m_brioullin.setVertex(8, a / 4, 0, a / 2);
		m_brioullin.setVertex(9, 0, a / 4, a / 2);
		m_brioullin.setVertex(10, -a / 4, 0, a / 2);
		m_brioullin.setVertex(11, 0, -a / 4, a / 2);

		m_brioullin.setVertex(12, a / 4, 0, -a / 2);
		m_brioullin.setVertex(13, 0, a / 4, -a / 2);
		m_brioullin.setVertex(14, -a / 4, 0, -a / 2);
		m_brioullin.setVertex(15, 0, -a / 4, -a / 2);

		m_brioullin.setVertex(16, a / 4, a / 2, 0);
		m_brioullin.setVertex(17, 0, a / 2, a / 4);
		m_brioullin.setVertex(18, -a / 4, a / 2, 0);
		m_brioullin.setVertex(19, 0, a / 2, -a / 4);

		m_brioullin.setVertex(20, a / 4, -a / 2, 0);
		m_brioullin.setVertex(21, 0, -a / 2, a / 4);
		m_brioullin.setVertex(22, -a / 4, -a / 2, 0);
		m_brioullin.setVertex(23, 0, -a / 2, -a / 4);

		PdVector x1 = new PdVector(-a / 4, a / 4, a / 4);
		PdVector y1 = new PdVector(a / 4, -a / 4, a / 4);
		PdVector z1 = new PdVector(a / 4, a / 4, -a / 4);

		PdVector t = new PdVector(3);

		int ind = 0;

		for (int i = -1; i <= 1; i++) {

			for (int j = -1; j <= 1; j++) {

				for (int k = -1; k <= 1; k++) {
					if ((i + j + k != 0) && i * j * k == 0
							|| (Math.abs(i + j + k) != 1 && i * j * k != 0)) {
						t = PdVector.blendNew(i, x1, j, y1, k, z1);
						m_element.setVertex(ind, t);
						ind++;
					}
				}
			}
		}
		m_element.setVertex(28, 3 / 8 * a, 3 / 8 * a, 3 / 8 * a);
		m_element.setVertexColor(28, Color.yellow);

		ind = 0;
		int x = 1;
		int y = 1;
		int z = 1;
		for (int i = 0; i < 6; i++) {
			PiVector line = m_edge.getPolygon(i);
			line.setSize(5);
			if (i % 2 == 0)
				x = -1;
			else
				x = 1;
			for (int j = 0; j <= 4; j++) {
				switch (j) {
				case 0: {
					y = -1;
					z = -1;
					break;
				}
				case 1: {
					y = -1;
					z = 1;
					break;
				}
				case 2: {
					y = 1;
					z = 1;
					break;
				}
				case 3: {
					y = 1;
					z = -1;
					break;
				}
				case 4: {
					y = -1;
					z = -1;
					break;
				}
				}
				if (i == 1 | i == 2) {
					t = PdVector.blendNew(x, x1, y, y1, z, z1);
				} else if (i == 3 | i == 4) {
					t = PdVector.blendNew(z, x1, x, y1, y, z1);
				} else if (i == 5 | i == 6) {
					t = PdVector.blendNew(y, x1, z, y1, x, z1);
				}
				line.m_data[j] = ind;
				m_edge.setVertex(ind, t);
				ind++;
			}
		}

		m_symmetry.getVertex(0).setName("\u0393");
		PdVector X = PdVector.blendNew(0.5, m_kdir.getVertex(2), 0.5,
				m_kdir.getVertex(3));
		m_symmetry.setVertex(1, X);
		m_symmetry.getVertex(1).setName("X");
		PdVector L = PdVector.blendNew(0.5, m_kdir.getVertex(1), 0.5,
				m_kdir.getVertex(2), 0.5, m_kdir.getVertex(3));
		m_symmetry.setVertex(2, L);
		m_symmetry.getVertex(2).setName("L");
		PdVector W = PdVector.blendNew(0.25, m_kdir.getVertex(1), 0.75,
				m_kdir.getVertex(2), 0.5, m_kdir.getVertex(3));
		m_symmetry.setVertex(3, W);
		m_symmetry.getVertex(3).setName("W");
		PdVector U = PdVector.blendNew(0.25, m_kdir.getVertex(1), 5. / 8.,
				m_kdir.getVertex(2), 5. / 8., m_kdir.getVertex(3));
		m_symmetry.setVertex(4, U);
		m_symmetry.getVertex(4).setName("U");
		PdVector K = new PdVector(3);
		K = PdVector.blendNew(3. / 8, m_kdir.getVertex(1), 3. / 4,
				m_kdir.getVertex(2), 3. / 8, m_kdir.getVertex(3));
		m_symmetry.setVertex(5, K);
		m_symmetry.getVertex(5).setName("K");

		m_Distance_gamma_1 = PdVector.dist(m_symmetry.getVertex(0),
				m_symmetry.getVertex(1));
		m_Distance_gamma_2 = PdVector.dist(m_symmetry.getVertex(0),
				m_symmetry.getVertex(2));
		m_Distance_gamma_3 = PdVector.dist(m_symmetry.getVertex(0),
				m_symmetry.getVertex(5));
		m_Distance_gamma_4 = PdVector.dist(m_symmetry.getVertex(0),
				m_symmetry.getVertex(3));
		m_Distance_gamma_5 = PdVector.dist(m_symmetry.getVertex(0),
				m_symmetry.getVertex(4));

	}

}
